<?php


function _process_uchoo($element, $form_state) {
  //for the preselect type, we use the 'select' widget with an option for each callback
  if ($element['#type'] == 'user_chooser_preselect') {
    $element['#options'] = module_invoke_all('user_chooser_preselect');
    $element['#theme'] = 'select';
  }
  //before anything, we have to work out how many users are valid
  elseif ($users = uchoo_get_users($element)) {
    //then we build the widget according to the valid user threshhold.
    if (count($users) < variable_get('user_chooser_threshhold', 50)) {
      $element['#theme'] = 'select';
      $element['#options'] = uchoo_display_all($users, TRUE);
      if (!$element['#multiple']) {
        $element['#options'] = array('' => t('Please choose...')) + $element['#options'];
      }
      $element = form_process_select($element);
      if (empty($element['#default_value']) && !empty($form_state['values'][$element['#name']])) {
        $element['#default_value'] = $form_state['values'][$element['#name']];
      }
      if (empty($element['#me'])) unset($element['#options'][$GLOBALS['user']->uid]);
    }
    else {
      $element['#theme'] = 'textfield';
      $element['#autocomplete_path'] = 'system/ajax/uchoo/'.$element['#type'].'/'.implode('/', $element['#args']);
      if (empty($element['#description'])) {
        $element['#description'] =  $element['#multiple'] ? t('Usernames, emails, or user IDs') : t('A username, email, or user ID');
      }
      if (empty($element['#default_value']) && !empty($form_state['values'][$element['#name']])) {
        //$uid = user_chooser_get_uid($element['#value']);
        //$element['#value'] = $uid ? user_load($uid)->name : '';
      }
    }
  }
  else {
    form_error($element, t("No users available to widget type '@type' with param @params", array(
      '@type' => $element['#type'],
      '@params' => implode(',', $element['#args'])
    )));
    return;
  }
  return $element;
}

//ajax menu callback
//first arg is the widget type, last arg is the string entered
function uchoo_autocomplete($type) {
  $args = func_get_args();
  array_shift($args);
  $string = array_pop($args);
  $pseudo_element = array(
    '#type' => $type,
    '#args' => $args
  );
  $users = uchoo_get_users($pseudo_element, $string, TRUE);
  $names = uchoo_display_all($users, FALSE);
  die(drupal_json_output($names));
}


/*
 * Settings form to set global restrictions
 */
function uchoo_config_form() {
  $form = array (
    'user_chooser_threshhold' => array(
      '#title' => t('Autocomplete threshhold'),
      '#description' => t('Below this number of items, a select box is shown, above it, an autocomplete field'),
      '#type' => 'textfield',
      '#size' => 3,
      '#maxlength' => 3,
      '#default_value' => variable_get('user_chooser_threshhold', 50),
      '#element_validate' => array('_element_validate_integer')
    ),
    'user_chooser_format' => array(
      '#title' => t('Display format'),
      '#description' => t('At least one of the following tokens must be used:') .implode(',', _uchoo_tokens()),
      '#type' => 'textfield',
      '#maxlength' => 64,
      '#default_value' => variable_get('user_chooser_format', '[user:name]'),
      '#element_validate' => array('uchoo_validate_format')
    ),
    'user_chooser_matching' => array(
      '#title' => t('Fields to match against'),
      '#description' => t('The autocomplete widget will get matches based on which fields?') .' '. t('Only applies to permission role and conditions based widgets.'),
      '#type' => 'checkboxes',
      '#options' => array(
        'u.uid' => t('User ID'),
        'u.name' => t('Username'),
        'u.mail' => t('Email'),
      ),
      '#default_value' => variable_get('user_chooser_matching', array('u.uid', 'u.name', 'u.mail')),
      '#weight' => 2
    ),
    'user_chooser_filters' => array(
      '#title' => t('Other global filters'),
      '#description' => t('Only applies to permission role and conditions based widgets.'),
      '#type' => 'checkboxes',
      '#options' => array(
        'status' => t('Exclude blocked users'),
        'authenticated' => t('Only allow authenticated users'),
      ),
      '#default_value' => variable_get('user_chooser_filters', array('status', 'authenticated')),
      '#weight' => 3
    )
  );
  return system_settings_form($form);
}

//form element callback
//checks that all submited values are in fact in the list.
function user_chooser_validate(&$element, &$form_state) {
  if ($element['#type'] == 'user_chooser_preselect') {
    $function = isset($element['#value']) ? $element['#value'] : $element['#default_value'];
    //replace the submitted callback with the values it returns
    $element['#value'] = $function($element, $form_state);
    if (empty($element['#me'])) {
      $key = array_search($GLOBALS['user']->uid, $element['#value']);
      if (is_integer($key)) unset($element['#value'][$key]);
    }
  }
  else {
    $values = is_array($element['#value']) ?  $element['#value'] : array($element['#value']);
    $uids = array();
    $invalids = array();
    foreach ($values as $key => $string) {
      $uid = user_chooser_get_uid($string);
      if (!$uid) {//users who don't exist
        $invalids[] = $string;
      }
      else{
        $values[$key] = $uid;
      }
    }
    $all_valid_users = uchoo_get_users($element);
    if (empty($element['#me'])) {
      $key = array_search($GLOBALS['user']->uid, $values);
      if (is_integer($key)) unset($values, $key);
    }
    foreach ($values as $uid) {
      if (empty($all_valid_users[$uid])) {
        //users who are filtered out
        $invalids[] = $uid;
      }
    }
    if (!empty($invalids)) {
      //owch I can't get the string for the invalid user here, I don't know where the element['#value'] changed
      form_error($element, t("Invalid user for field: @name", array('@name' => $element['#name'])));
      //set the contents of the field back to the errant value ready for the form rebuilding
      $element['#value'] = $form_state['values'][$element['#name']];
      return;
    }
    //if it passes validation, set the #value to the uid, which is usually expected.
    //however, this looks odd if some other form element fails validation
    $element['#value'] = is_array($element['#value']) ? $values : $values[0];
  }
  form_set_value($element, $element['#value'], $form_state);
}

//retrives an array of user ids according to the form widget parameters and optionally the string from the autocomplete field.
//note that the $args may be keyed, esp for user_chooser_conditions
//in a perfect world, the $element would been cached perhaps in the $form_state or {cache_form} table and we would have access to it here,
//instead, we are passing $args back and forth, and must deal with the #me elsewhere.
function uchoo_get_users($element, $string = '', $limit = FALSE) {
  //get the relevent settings
  $filters = variable_get('user_chooser_filters', array('status', 'authenticated'));
  $matching = variable_get('user_chooser_matching', array('u.uid', 'u.name', 'u.mail'));

  //deal with the types which do their own db queries
  switch ($element['#type']) {
    case 'user_chooser_callback':
    case 'user_chooser_preselect':
      //first arg has to be the callback function
      $callback = reset($element['#args']);
      $args = array_splice($element['#args'], 1);
      return call_user_func($callback, $args);
    case 'user_chooser_view':
      drupal_set_message("user_chooser_view' hasn't been written yet");
  }

  //if we're still here, the rest of the options use a query
  $query = db_select('users', 'u');
  if (strlen($string)) {
    $or = db_or();
    foreach ($matching as $field) {
      $query->condition($or->condition($field, db_like($string) . '%', 'LIKE'));
    }
  }
  if (in_array('status', $filters)) {
    $query->condition('u.status', TRUE);
  }
  if (in_array('authenticated', $filters)) {
    $query->condition('u.uid', 0, '!=');
  }
  switch ($element['#type']) {
    case 'user_chooser_permissions':
      $rids = array();
      foreach($element['#args'] as $perm) {
        $roles = user_roles(TRUE, $perm);
        foreach ($roles as $rid => $name);
        $rids[] = $rid;
      }
      if (!count($element['#args'])) continue;
      $element['#args'] = array_unique($rids);
      //then use the roles method
    case 'user_chooser_roles':
      $pos = array_search(DRUPAL_AUTHENTICATED_RID, $element['#args']);
      if (is_numeric($pos)) { //special case for role 2
        unset($element['#args'][$pos]);
      }
      if (count($element['#args'])) {
        $query->join('users_roles', 'ur', 'ur.uid = u.uid');
        $query->condition('ur.rid', $element['#args']);
      }
      break;
    default:
      drupal_set_message('Unsupported user_chooser element #type:'. $element['#type']);
  }
  if ($limit) {
    $query->range(0, variable_get('user_chooser_threshhold', 50));
  }
  return $query->fields('u', array('uid', 'name', 'mail'))->execute()->fetchAllAssoc('uid');
}


/*
 * validation callback for settings form element
 */
function uchoo_validate_format(&$element, $form_state) {
  foreach (_uchoo_tokens() as $token) {
    if (is_numeric(strpos($element['#value'], $token))) return;
  }
  form_error($element, t('At least one of the following tokens must be used:') .implode(',', _uchoo_tokens()));
}



/*
 * NOT an implementation of hook_tokens, just a local utility
 */
function _uchoo_tokens() {
  return array('[user:name]', '[user:mail]', '[user:uid]');
}


//takes an array of pseudo users and formats them for display
function uchoo_display_all($users, $use_uid = TRUE) {
  $template = variable_get('user_chooser_format', '[user:name]');
  //convert the vars in the template to token-like strings
  //$accounts = user_load_multiple($uids);
  $names = array();
  foreach ($users as $uid => $account) {
    $name = token_replace($template, array('user' => $account));
    $names[$uid] = $name;
  }
  if ($use_uid) return $names;
  return drupal_map_assoc($names);
}


function user_chooser_get_uid($string) {
  return db_select('users', 'u')
  ->fields('u', array('uid'))
  ->condition('uid', 0, '!=')
  ->condition(db_or()
    ->condition('uid', $string)
    ->condition('name', $string)
    ->condition('mail', $string)
  )
  ->execute()
  ->fetchField();
}

